
/* Project      tie
 * Author       TriX
 * Description  Converts a DoorsOS program (DoorsOS final beta 2) to a [tie]
 *              program (.92p file
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define  ulong  unsigned long

/* Function ReadFile:  Reads the whole file into memory */
int ReadFile (char* File, ulong *filesize, int offset, unsigned char **Pointer)
{
   FILE *tmp;
   int i;

   if ((tmp = fopen (File, "rb")) == NULL)
   {
     printf ("Can't open file %s\n", File);
     exit (-1);
   }
   fseek (tmp, 0, SEEK_END);
   *filesize = ftell (tmp);
   fseek (tmp, offset, SEEK_SET);
   *Pointer = (unsigned char*) malloc (*filesize);
   i = 0;
   while (!feof (tmp))
   {
      (*Pointer)[i] = fgetc (tmp);
      i++;
   } /* while */
   fclose (tmp);
   return 1;

} /* ReadFile */

int main (int argc, char *argv[])
{
   unsigned char *InputFile, OutFile[32];
   ulong    Size, i, pos, FSize, csum;
   FILE    *out;

   if (argc != 2)
   {
      printf ("Usage:  convert  <doorsprgm.9xz>\n\n");
      return 1;
   } // if

   i = 0;
   while (argv[1][i] != '.')
   {
      OutFile[i] = argv[1][i];
      OutFile[i + 1] = '\0';
      i++;
   } // while

   strcat (OutFile, ".92p");
   ReadFile (argv[1], &Size, 0, &InputFile);
   out = fopen (OutFile, "wb");
   fprintf (out, "**TI92**");

   pos = 8;
   for (i = 0; i < 64; i++) fputc (InputFile[pos + i], out);
   pos += 64;
   fputc (0x12, out);
   pos++;
   for (i = 0; i < 3; i++) fputc (InputFile[pos + i], out);
   pos += 3;
   FSize = (InputFile[pos]) + (InputFile[pos + 1] << 8);
   FSize += 11;
   fputc (FSize & 0xff, out);
   fputc ((FSize >> 8) & 0xff, out);
   pos += 2;
   for (i = 0; i < 8; i++) fputc (InputFile[pos + i], out);
   pos += 8;
   FSize = (InputFile[pos] << 8) + (InputFile[pos + 1]);
   FSize += 11;
   fputc ((FSize >> 8) & 0xff, out);
   csum = (FSize >> 8) & 0xff;
   fputc (FSize & 0xff, out);
   csum += (FSize & 0xff);
   pos += 2;
   for (i = pos; i < Size - 3; i++)
   {
      fputc (InputFile[i], out);
      csum += InputFile[i];
   } // for

   fputc (0xe9, out);
   fputc (0x12, out);
   fputc (0xe4, out);
   fputc (0x00, out);
   fputc (0xe8, out);
   fputc (0x19, out);
   fputc (0xe4, out);
   fputc (0xe5, out);
   fputc (0x00, out);
   fputc (0x00, out);
   fputc (0x40, out);
   fputc (0xdc, out);
   csum += 0xe9 + 0x12 + 0xe4 + 0xe8 + 0x19 + 0xe4 + 0xe5 + 0x40 + 0xdc;

   fputc (csum & 0xff, out);
   fputc ((csum << 8) & 0xff, out);
   fclose (out);
   free (InputFile);

   return 0;
} /* main */
