
/*
 *  Project     [tie]
 *  Author      TriX
 *  Description TI92 linker for the parallel & serial cable
 *  Module      Main
 */

#include "ti92.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int SendHeader (TIHeader tih)
{
   if (!SendByte (0x9)) return 0;
   SendByte (tih.Type & 0xff);
   SendByte ((tih.Stuff >> 8) & 0xff);
   SendByte (tih.Stuff & 0xff);
   return 1;
} /* SendHeader */

int SendHInts (int type, int stuff)
{
   if (!SendByte (0x9)) return 0;
   SendByte (type & 0xff);
   SendByte ((stuff >> 8) & 0xff);
   SendByte (stuff & 0xff);
   return 1;
}

int GetChar ()
{
   unsigned char tmp;

   GetByte (&tmp);

   return tmp;
}


int GetWord ()
{
   unsigned char tmp[2];

   GetByte (&tmp[0]);
   GetByte (&tmp[1]);

   return (tmp[1] << 8) + (tmp[0]);   /* Conversion to the right format */
}


long GetDWord ()
{
   unsigned char tmp[4];

   GetByte (&tmp[0]);
   GetByte (&tmp[1]);
   GetByte (&tmp[2]);
   GetByte (&tmp[3]);

   return (tmp[3] << 24) + (tmp[2] << 16) + (tmp[1] << 8) + (tmp[0]);
}


int GetHeader (TIHeader *tih)
{
   unsigned char tmp;
   int k;

   if (!GetByte (&tmp)) return 0;
   k = tmp;
   if (k != 0x89) return 0;

   tih->Type = GetChar ();
   tih->Stuff = GetWord ();

   return 1;
} /* GetHeader */

int GetBody (char *savewhere, int *size)
{
   char  *myptr, tmp;
   int   i;

   TIHeader tih;

   if (!GetHeader (&tih))  return 0;
   if (tih.Type != 0x15)   return 0;

   GetChar (); GetChar (); GetChar (); GetChar ();

   myptr = savewhere;
   if (size != NULL) *size = tih.Stuff - 4;

   for (i = 0; i < tih.Stuff - 4; i++)
   {
      GetByte (myptr);
      myptr++;
   } /* for */

   GetByte (&tmp); GetByte (&tmp);          /* Ignore checksum */

   return 1;
} /* GetBody */

int SendVarHeader (VarHeader vh)
{
   int HSize = 6 + vh.Namelength, chksum, i;

   SendHInts (0xa2, ((HSize & 0xff) << 8) + ((HSize >> 8) & 0xff));
   chksum = 0;

   SendByte ((vh.VariableSize >> 0)  & 0xff);
   SendByte ((vh.VariableSize >> 8)  & 0xff);
   SendByte ((vh.VariableSize >> 16) & 0xff);
   SendByte ((vh.VariableSize >> 24) & 0xff);

   chksum += (vh.VariableSize >> 0)  & 0xff;
   chksum += (vh.VariableSize >> 8)  & 0xff;
   chksum += (vh.VariableSize >> 16) & 0xff;
   chksum += (vh.VariableSize >> 24) & 0xff;

   SendByte (vh.VariableType);
   SendByte (vh.Namelength);

   for (i = 0; i < vh.Namelength; i++)
   {
      SendByte (vh.VariableName[i]);
      chksum += vh.VariableName[i];
   }

   chksum += (vh.VariableType & 0xff);
   chksum += (vh.Namelength   & 0xff);

   /* Send chechsum */
   SendByte ((chksum >> 0) & 0xff);
   SendByte ((chksum >> 8) & 0xff);

   return 0;
}

int GetVarHeader (VarHeader *Savein)
{
   int i;

   Savein->VariableSize = GetDWord ();
   Savein->VariableType = GetChar ();
   Savein->Namelength   = GetChar ();

   for (i = 0; i < Savein->Namelength; i++)
      Savein->VariableName[i] = GetChar ();

   Savein->VariableName[i] = '\0';

   Savein->Checksum = GetWord ();

   return 1;
}


int GetScreenDump (char *Filename)
{
   char *screendump;
   int size, i;
   FILE *savedump;
   TIHeader tih;

   if (!SendHInts (0x6d, 0)) return 0;
   GetHeader (&tih);
   if (tih.Stuff != 0) return 0;

   /* Receive loop */
   printf ("Receiving screen dump\n");
   screendump = (char *) malloc (30 * 130);

   if (!GetBody (screendump, &size)) return 0;
   SendHInts (0x56, 0);               /* ACK */

   /* Save screen dump */
   savedump = fopen (Filename, "w");
   for (i = 0; i < size; i++)
   {
       fputc ((screendump[i] & 128) == 0, savedump);
       fputc ((screendump[i] & 64) == 0, savedump);
       fputc ((screendump[i] & 32) == 0, savedump);
       fputc ((screendump[i] & 16) == 0, savedump);
       fputc ((screendump[i] & 8) == 0, savedump);
       fputc ((screendump[i] & 4) == 0, savedump);
       fputc ((screendump[i] & 2) == 0, savedump);
       fputc ((screendump[i] & 1) == 0, savedump);
   }

   fclose (savedump);
   free (screendump);

   return 1;
} /* if */


int GetBackup (char *FileName)
{
    long BackupSize = 0, i;

    TIHeader  Comm;
    VarHeader VComm;

    FILE *Destination;

    char *Memory = (char *) malloc (256 * 1024); /* Max backupsize 256kB */
    VComm.VariableSize = 0x00;
    VComm.VariableType = 0x1d;
    strcpy (VComm.VariableName, "main\\backup");
    VComm.Namelength   = 11;

    SendVarHeader (VComm);  /* Request a backup */

    if (!GetHeader (&Comm)) return 0;
    if (Comm.Stuff != 0) return 0;

    /* Get variable header */
    while (1)
    {
       GetHeader (&Comm);
       if (Comm.Type == 0x92) break; /* End of backup */

       GetVarHeader (&VComm);

       SendHInts (0x56, 0x0000);
       SendHInts (0x09, VComm.VariableSize);

       GetHeader (&Comm);
       if (Comm.Stuff != 0) break;

       GetHeader (&Comm);

       GetBody (&Memory[BackupSize], NULL);
       BackupSize += VComm.VariableSize;

       SendHInts (0x56, 0x0000);
       printf ("\rReceived %ld bytes", BackupSize);
    } /* while */

    printf ("\rReceived %ld bytes\n", BackupSize);
    SendHInts (0x56, 0x0000);

    Destination = fopen (FileName, "wb");
    if (Destination == NULL)
    {
       printf ("Can't open backupfile\n");
       return 0;
    }

    for (i = 0; i < BackupSize; i++)
       fputc (Memory[i], Destination);

    fclose (Destination);

    return 1;
} /* GetBackup */

int ProcessArguments (int argc, char *argv[])
{
   int i = 1;

   while (i < argc)
   {
      if (argv[i][0] != '-')
      {
         printf ("Ignored parameterswitch %s\n", argv[i]);
         i++;
         continue;
      }

      switch (argv[i][1])
      {
         case 'b':  if (!GetBackup (argv[i + 1]))
                       printf ("Can't receive backup\n");
                    i++;
                    break;
         case 's':  if (!GetScreenDump (argv[i + 1]))
                       printf ("Can't get screen dump\n");
                    i++;
                    break;
         default:   printf ("Ignored parameterswitch %s\n", argv[i]);
                    break;
      }

      i++;
   }
   return 0;
}

int main (int argc, char *argv[])
{
   printf ("ti92link by MXM Projects\n");

   SetPort (LPT_PORT_1);

   if (argc > 1)
      ProcessArguments (argc, argv);
   else
      printf ("No parameters\n");

   ClosePort ();

   return 0;
} /* main */
